/*--------------------------------------------------------------
    exrsh :  Extremely Restricted Shell :)  v1.0
      Written  by H.Goto, Jun 4, 2002
      http://www.sc.isc.tohoku.ac.jp/~hgot/
--------------------------------------------------------------*/

#define		MOTDFILE	"/etc/motd"

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<signal.h>

#define		LINEBUFLEN	80
#define		MAXCMDLEN	80
#define		MAXCMDNAMELEN	16
#define		MAXPATHLEN	64

static char	linebuf[LINEBUFLEN+1];
static char	cmdbuf[MAXCMDLEN+1];

typedef struct {
	char	name[MAXCMDNAMELEN];
	char	path[MAXPATHLEN];
} CmdTable;

static CmdTable	cmdtable[] = {\
	{ "logout", "" },
	{ "exit", "" },
	{ "help", "" },
	{ "w", "/usr/bin/w" },
	{ "who", "/usr/bin/who" },
	{ "passwd", "/usr/bin/passwd" },
/*	{ "yppasswd", "/usr/bin/yppasswd" },
*/
	{ "", "" },
};




/*--------------------------------------------------------------
    Show motd.
--------------------------------------------------------------*/

static int show_motd(){
	FILE	*fp;
	int	c;
	struct stat	filestat;

	if ( -1 == stat(MOTDFILE,&filestat) )  return(1);
	if ( NULL == (fp = fopen(MOTDFILE,"r")) )  return(2);
	while ( EOF != (c = fgetc(fp)) )  fputc(c,stdout);
	fclose(fp);
	return(0);
}




/*--------------------------------------------------------------
    Show help message.
--------------------------------------------------------------*/

static void show_help(){
	int	i;
	fputs("This is \"exrsh\", the Extremely Restricted Shell.\n",stdout);
	fputs("Available commands: ",stdout);
	for ( i=0 ; strlen(cmdtable[i].name) != 0 ; i++ ){
		if ( i != 0 )  fputs(", ",stdout);
		fputs(cmdtable[i].name,stdout);
	}
	fputs("\n",stdout);
	fputs("Arguments will not be passed to the command invoked. \n",stdout);
}




/*--------------------------------------------------------------
    Issue command.
--------------------------------------------------------------*/

static int issue_command(char *cmdname){
	int	i;
	if ( 0 == strlen(cmdname) )  return(0);	/* null command */
	if ( 0 == strcmp(cmdname, "logout") )  return(1);
	if ( 0 == strcmp(cmdname, "exit") )  return(1);
	if ( 0 == strcmp(cmdname, "help") ){
		show_help();
		return(2);
	}
	for ( i=0 ; strlen(cmdtable[i].name) != 0 ; i++ ){
		if ( 0 == strcmp(cmdtable[i].name, cmdname) )  break;
	}
	if ( strlen(cmdtable[i].name) == 0 ){
		/* unknown command */
		return(-1);
	}
	strcpy(cmdbuf,cmdtable[i].path);
	system(cmdbuf);
	return(0);
}




/*--------------------------------------------------------------
    Main routine
--------------------------------------------------------------*/

int main(int ac,char *av[]){
	int	i;
	int	len,pos;
	int	loop_exit;

	for ( i=1 ; i<ac ; i++ ){
		if ( 0 == strcmp(av[i],"-c") ){
			/* Exit immediately if -c is specified. */
			exit(0);
		}
		if ( 0 == strcmp(av[i],"-h") || 0 == strcmp(av[i],"-help") ){
			fputs("Extremely Restricted Shell (exrsh) v1.0 \n",stdout);
			fputs("Copyright (C) 2002 Hideaki Goto \n\n",stdout);
			fputs("usage: exrsh [-c cmd] \n",stdout);
			fputs("    exrsh accept only a limited set of commands. \n",stdout);
			fputs("    The option -c is ignored. \n",stdout);
			return(0);
		}
		fputs("Undefined option.\n",stderr);
		exit(1);
	}

	show_motd();
	show_help();

	/* ignore CTRL+C */
	signal(SIGINT,SIG_IGN);

	loop_exit = 0;
	while(1){
		fputs("exrsh>> ",stdout);
		if ( NULL == fgets(linebuf,LINEBUFLEN,stdin) )  break;
		if ( 0 == (len = strlen(linebuf)) )  continue;
		/* skip the leading white spaces */
		for ( i=pos=0 ; i<len ; i++ ){
			if ( linebuf[i] == ' ' ){  pos++;  continue; }
			if ( linebuf[i] == '\t' ){  pos++;  continue; }
			break;
		}
		/* find the end point of the command name */
		for ( i=pos ; i<len ; i++ ){
			if ( linebuf[i] <= ' ' ){
				linebuf[i] = 0;  break;
			}
		}
		switch ( issue_command(linebuf+pos) ){
		  case 1:
			loop_exit = -1;
			break;
		  case -1:
			fprintf(stderr,"%s: Command not found.\n",linebuf+pos);
			break;
		}

		/* skip the remaining strings */
		while( 0x0a != linebuf[len-1] && 0 != linebuf[len-1] ){
			if ( NULL == fgets(linebuf,LINEBUFLEN,stdin) )  break;
			if ( 0 == (len = strlen(linebuf)) ){
				loop_exit = -1;
				break;
			}
		}
		if ( loop_exit )  break;
	}
	return(0);
}


